/*  BotikBalance - User's account balance informer for Botik Network 
 *                 for Windows 9x/ME/NT/2000/XP/2003
 *  Copyright (C) 2005 Alexander V. Melnikoff
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Sorry for the long lines in the sources, I have wide screen... ;-)
 */

#include <stdafx.h>
#include <bbobj.h>
#include <botiknadminobj_h.h>
#include <classfactory.h>

// Global variables.
HMODULE		g_hModule		= NULL;
LONG		g_lDllRefCnt	= 0;
CDataCont*	g_pDataCont		= NULL;

// Forward declaration.
BOOL RegisterServer(const CLSID& clsid_, LPTSTR szObjName_);
BOOL UnRegisterServer(const CLSID& _clsid);
BOOL RegisterProgID(LPTSTR szProgID_, LPTSTR szObjName_, LPTSTR szVer_, const CLSID& clsid_, LPTSTR szCurVer_);
BOOL UnRegisterProgID(LPTSTR szProgID_, LPTSTR szObjName_, LPTSTR szVer_);

BOOL APIENTRY DllMain(HMODULE hModule_, DWORD dwReason_, LPVOID)
{
	cfdprintf(_T("DllMain"));

	switch (dwReason_)
	{
	case DLL_PROCESS_ATTACH: 
		cfdprintf("DllMain: DLL_PROCESS_ATTACH");
		g_hModule = hModule_;
		break;
	case DLL_THREAD_ATTACH:
		cfdprintf("DllMain: DLL_THREAD_ATTACH");
		break;
	case DLL_THREAD_DETACH:
		cfdprintf("DllMain: DLL_THREAD_DETACH");
		break;
	case DLL_PROCESS_DETACH:
		cfdprintf("DllMain: DLL_PROCESS_DETACH");
		break;
	};

	cfdprintf(_T("!DllMain"));

    return(TRUE);
};

STDAPI DllCanUnloadNow(void)
{
	cfdprintf(_T("DllCanUnloadNow: DllRefCnt: %d"), g_lDllRefCnt);

	return(g_lDllRefCnt ? S_FALSE : S_OK);
};

STDAPI DllGetClassObject(REFCLSID rCLSID_, REFIID rIID_, LPVOID *ppReturn_) 
{
	HRESULT			hr = E_FAIL;
	CClassFactory*	pClassFactory;

	cfdprintf(_T("DllGetClassObject"));

	*ppReturn_ = NULL;

	// If this DLL doesn't support requested clsid return appropriate error code.
	if(rCLSID_ != CLSID_BotikBalance) 
	{
		cfdprintf(_T("DllGetClassObject: DLL doesn't support specified CLSID"));
		hr = CLASS_E_CLASSNOTAVAILABLE;
	}
	else
	{
		// Create the CClassFactory object.
		pClassFactory = new CClassFactory(rCLSID_);
		if(pClassFactory == NULL) 
		{
			cfdprintf(_T("DllGetClassObject: Create CClassFactory failed"));
			hr = E_OUTOFMEMORY;
		}
		else
		{
			// Query requested interface.
			hr = pClassFactory->QueryInterface(rIID_, ppReturn_);
			pClassFactory->Release();
		};
	};

	cfdprintf(_T("!DllGetClassObject"));

	return(hr);
};

STDAPI DllRegisterServer(void) 
{
	HRESULT	hr = SELFREG_E_CLASS;
	
	cfdprintf(_T("DllRegisterServer"));

	__try
	{
		// Register server.
		if(!RegisterServer(CLSID_BotikBalance, BBOBJ_OBJ_NAME))
			__leave;

		// Register ProgIDs for each version.
		if(!RegisterProgID(BBOBJ_PROG_ID, BBOBJ_OBJ_NAME, _T("1"), CLSID_BotikBalance, NULL))
			__leave;

		// Register default ProgID.
		if(!RegisterProgID(BBOBJ_PROG_ID, BBOBJ_OBJ_NAME, NULL, CLSID_BotikBalance, _T("1")))
			__leave;

		hr = S_OK;
	}

	__finally
	{
		if(FAILED(hr))
		{
			cfdprintf(_T("DllRegisterServer: Failed (0x%x)"), hr);
			UnRegisterServer(CLSID_BotikBalance);
			UnRegisterProgID(BBOBJ_PROG_ID, BBOBJ_OBJ_NAME, _T("1"));
			UnRegisterProgID(BBOBJ_PROG_ID, BBOBJ_OBJ_NAME, NULL);
		};
	};

	cfdprintf(_T("!DllRegisterServer"));

	return(hr);
};

STDAPI DllUnregisterServer(void) 
{
	UnRegisterServer(CLSID_BotikBalance);

	UnRegisterProgID(BBOBJ_PROG_ID, BBOBJ_OBJ_NAME, _T("1"));
	UnRegisterProgID(BBOBJ_PROG_ID, BBOBJ_OBJ_NAME, NULL);

	return(S_OK);
};

// Assistant functions...
BOOL RegisterServer(const CLSID& clsid_, LPTSTR szObjName_) 
{
	USES_CONVERSION;

	HKEY	hKey	= NULL;
	HKEY	hSubKey = NULL;
	BOOL	fOk		= FALSE;

	__try
	{
		LONG	lRet;
		TCHAR	szCLSID[MAX_PATH+1];
		TCHAR	szKeyName[MAX_PATH+1];
		TCHAR	szPath[MAX_PATH+1];
		LPWSTR	pwsz;

		// Convert clsid to wide string.
		StringFromIID(clsid_, &pwsz);
		if(!pwsz) __leave;
		_tcscpy(szCLSID, W2T(pwsz));
		CoTaskMemFree(pwsz);

		StringCchPrintf(szKeyName, MAX_PATH, _T("CLSID\\%s"), szCLSID);
		lRet = RegCreateKeyEx(HKEY_CLASSES_ROOT, szKeyName, 0, NULL, 
							  REG_OPTION_NON_VOLATILE, KEY_WRITE, 
							  NULL, &hKey, NULL);
		cfCHECK_RET_L(lRet);

		lRet = RegSetValueEx(hKey, NULL, 0, REG_SZ, (LPBYTE)szObjName_, 
							(lstrlen(szObjName_)+1)*sizeof(TCHAR));
		cfCHECK_RET_L(lRet);

		lRet = RegCreateKeyEx(hKey, _T("InprocServer32"), 0, NULL, 
						  REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, 
						  &hSubKey, NULL);
		cfCHECK_RET_L(lRet);

		GetModuleFileName((HMODULE)g_hModule, szPath, MAX_PATH);

		lRet = RegSetValueEx(hSubKey, NULL, 0, REG_SZ, (LPBYTE)szPath, 
							(lstrlen(szPath)+1)*sizeof(TCHAR));
		cfCHECK_RET_L(lRet);

		lRet = RegSetValueEx(hSubKey, _T("ThreadingModel"), 0, REG_SZ, 
							(LPBYTE)_T("Apartment"), 
							(lstrlen(_T("Apartment"))+1)*sizeof(TCHAR));
		cfCHECK_RET_L(lRet);

		fOk = TRUE;
	}

	__finally
	{
		if(hSubKey) RegCloseKey(hSubKey);
		if(hKey) RegCloseKey(hKey);
	};

	return(fOk);
};

BOOL UnRegisterServer(const CLSID& clsid_) 
{
	TCHAR	szCLSID[MAX_PATH+1];
	TCHAR	szSubKey[MAX_PATH+1];
	LONG	lRet;
	LPWSTR	pwsz;

	// Convert clsid to wide string.
	StringFromIID(clsid_, &pwsz);
	if(!pwsz) return(FALSE);
	_tcscpy(szCLSID, CW2T(pwsz));
	CoTaskMemFree(pwsz);

	StringCchPrintf(szSubKey, MAX_PATH, _T("CLSID\\%s\\InprocServer32"), szCLSID);
	lRet = RegDeleteKey(HKEY_CLASSES_ROOT, szSubKey);
	cfCHECK_RET(lRet);

	StringCchPrintf(szSubKey, MAX_PATH, _T("CLSID\\%s"), szCLSID);
	lRet = RegDeleteKey(HKEY_CLASSES_ROOT, szSubKey);
	cfCHECK_RET(lRet);

	return(TRUE);
};

BOOL RegisterProgID(LPTSTR szProgID_, LPTSTR szObjName_, LPTSTR szVer_, const CLSID& clsid_, LPTSTR szCurVer_)
{
	USES_CONVERSION;

	HKEY	hKey	= NULL;
	HKEY	hSubKey = NULL;
	BOOL	fOk		= FALSE;

	__try
	{
		LONG	lRet;
		TCHAR	szCLSID[MAX_PATH+1];
		TCHAR	szKeyName[MAX_PATH+1];
		LPWSTR	pwsz;

		// Convert clsid to wide string.
		StringFromIID(clsid_, &pwsz);
		if(!pwsz) __leave;
		_tcscpy(szCLSID, W2T(pwsz));
		CoTaskMemFree(pwsz);

		if(szVer_)
			StringCchPrintf(szKeyName, MAX_PATH, _T("%s.%s.%s"), szProgID_, szObjName_, szVer_);
		else
			StringCchPrintf(szKeyName, MAX_PATH, _T("%s.%s"), szProgID_, szObjName_);
		lRet = RegCreateKeyEx(HKEY_CLASSES_ROOT, szKeyName, 0, NULL, 
								REG_OPTION_NON_VOLATILE, KEY_WRITE, 
								NULL, &hKey, NULL);
		cfCHECK_RET_L(lRet);

		lRet = RegSetValueEx(hKey, NULL, 0, REG_SZ, (LPBYTE)szObjName_, 
								(lstrlen(szObjName_)+1)*sizeof(TCHAR));
		cfCHECK_RET_L(lRet);

		lRet = RegCreateKeyEx(hKey, _T("CLSID"), 0, NULL, 
						REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, 
						&hSubKey, NULL);
		cfCHECK_RET_L(lRet);

		lRet = RegSetValueEx(hSubKey, NULL, 0, REG_SZ, (LPBYTE)szCLSID, 
								(lstrlen(szCLSID)+1)*sizeof(TCHAR));
		cfCHECK_RET_L(lRet);

		if(szCurVer_)
		{
			RegCloseKey(hSubKey);
			lRet = RegCreateKeyEx(hKey, _T("CurVer"), 0, NULL, REG_OPTION_NON_VOLATILE, 
								  KEY_WRITE, NULL, &hSubKey, NULL);
			cfCHECK_RET_L(lRet);

			StringCchPrintf(szKeyName, MAX_PATH, _T("%s.%s.%s"), szProgID_, szObjName_, szCurVer_);
			lRet = RegSetValueEx(hSubKey, NULL, 0, REG_SZ, (LPBYTE)szKeyName, 
								(lstrlen(szKeyName)+1)*sizeof(TCHAR));
			cfCHECK_RET_L(lRet);
		};

		fOk = TRUE;
	}

	__finally
	{
		if(hSubKey) RegCloseKey(hSubKey);
		if(hKey) RegCloseKey(hKey);
	};

	return(fOk);
};

BOOL UnRegisterProgID(LPTSTR szProgID_, LPTSTR szObjName_, LPTSTR szVer_)
{
	TCHAR	szKeyName[MAX_PATH+1];
	LONG	lRet;

	if(szVer_)
		StringCchPrintf(szKeyName, MAX_PATH, _T("%s.%s.%s\\CLSID"), szProgID_, szObjName_, szVer_);
	else
		StringCchPrintf(szKeyName, MAX_PATH, _T("%s.%s\\CLSID"), szProgID_, szObjName_);
	lRet = RegDeleteKey(HKEY_CLASSES_ROOT, szKeyName);
	cfCHECK_RET(lRet);

	if(szVer_)
		StringCchPrintf(szKeyName, MAX_PATH, _T("%s.%s.%s\\CurVer"), szProgID_, szObjName_, szVer_);
	else
		StringCchPrintf(szKeyName, MAX_PATH, _T("%s.%s\\CurVer"), szProgID_, szObjName_);
	lRet = RegDeleteKey(HKEY_CLASSES_ROOT, szKeyName);

	if(szVer_)
		StringCchPrintf(szKeyName, MAX_PATH, _T("%s.%s.%s"), szProgID_, szObjName_, szVer_);
	else
		StringCchPrintf(szKeyName, MAX_PATH, _T("%s.%s"), szProgID_, szObjName_);
	lRet = RegDeleteKey(HKEY_CLASSES_ROOT, szKeyName);
	cfCHECK_RET(lRet);

	return(TRUE);
};